#ifndef ___forb__
#define ___forb__

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include <unistd.h>
#include <math.h>

#include "stock.h"
#include "orb.h"
#include "coords.h"
#include "fake.h" 
#include "forb.h"
#include <regex.h> 
#include "tr.h" 
#include "Pkt.h"

#define FORBMAGIC "#orb" 
#define NFORBMAGIC 4
#define NMATCHES 1
#define REPSIZE 16

#define SEGMENT_SECONDS 3600

#define END_SEGMENT  -2
#define END_FORB     -1
#define FORB_ERROR   -3
#define LASTPACKET   "LastSegmentPacket" 
#define LASTPACKET_SIZE 64

#define MAXPKTID    2147483000	       /* need some cushion to make sure time
				        * advances to new segment */

typedef struct ForbFlags { 
    unsigned int verbose     : 2 ; 
    unsigned int synchronous : 1 ; /* release packets at approximate 
				    rate dictated by time tags */
    unsigned int current     : 1 ; /* adjust time tags to appear current */
    unsigned int write	     : 1 ; /* set if write permission ; unset if read */
    unsigned int written     : 1 ; /* set if any packets written */
    unsigned int newsegment  : 1 ; /* set to force opening a new segment file */
} ForbFlags ;

typedef struct Forbfile {
    char *dirname ; 		/* name of directory when using multiple segment files */
    char *filename ; 		/* name of current forbfile */
    FILE *file ; 		/* file associated with filename */
    int id ; 			/* id for this forbfile */
    int fd ; 			/* associated file descriptor */
    char representation[REPSIZE] ; /* representation to use when writing new packets */
    Tbl *index ;		/* index of packets read from current filename*/

    char *select ; 		/* select character string */
    char *reject ;		/* reject character string */
    regex_t *select_pgm ; 	/* select program */
    regex_t *reject_pgm ;	/* reject program */
    ForbFlags flags ; 		/* various flags */

    double tdelta ; 		/* time difference between packet time and now */

    int first_pktid ; 		/* first pktid in this file */
    int first_time ; 		/* integer truncated first time in this file */
    int segment_period ; 	/* maximum segment time */
    double maxsize ; 		/* maximum size of forb directory */
    int maxpkts ; 		/* approximate maximum number of packets in segment file */
    int max_segment_size	;	/* approximate maximum size of segment file */
    int max_time_gap ;		/* save packets with times this far in future, but don't account for them */
    int maxpktid ; 		/* maximum packet id */
    int maxskip ; 		/* maximum # packets to allow after #packets considerations 
				      before forcing new segment file */
    int count ;			/* only allow skipping MAXSKIP packets before forcing a new segment file, 
				      regardless of considerations about time */
    int segment_bytes ; 	/* approximate byte count for current segment */

    int pktid ; 		/* current pktid */
    char srcname[ORBSRCNAME_SIZE] ; /* current srcname */
    double time ;		/* current time */
    double maxtime ; 		/* current maximum time on a packet */
    int nbytes ; 		/* current #bytes */
    char *packet ;		/* current packet */
    int bufsize ; 		/* packet buffer size */

} Forbfile ; 

typedef struct ForbPkt {
    int pktid ; 
    long fpos ; 
    double time ; 
    char srcname[ORBSRCNAME_SIZE] ; 
} ForbPkt ; 

extern int Save_time ;

#ifdef  __cplusplus
extern "C" {
#endif

extern Forbfile * fd2forb ( int fd, Forbfile *aforb );
extern int forbopen ( char *filename, char *representation );
extern int forbclose ( int orb );
extern int forbselect ( int orb, char *select );
extern int forbreject ( int orb, char *reject );
extern int forbseek ( int orb, int which );
extern int forbafter ( int orb, double atime );
extern int forbtell ( int orb );
extern int forbget ( int orb, int which, int *rpktid, char *srcname, double *time, char **packet, int *nbytes, int *bufsize );
extern int forbreap ( int orb, int *rpktid, char *srcname, double *time, char **packet, int *nbytes, int *bufsize );
extern int forbput ( int orb, char *srcname, double time, char *packet, int nbytes );
extern char * orbpkt_string ( char *srcname, double time, char *packet, int nbytes );

#ifdef  __cplusplus
}
#endif

#endif
